//Copyright (c) Dmitry Kramkov, 2000-2018. All rights reserved. 

#ifndef __Exam_hpp__
#define __Exam_hpp__

#include "cfl/Data.hpp"
#include "cfl/AssetModel.hpp"
#include "cfl/InterestRateModel.hpp"
#include <numeric>

/**
 * \file   Exam.hpp
 * \author Dmitry Kramkov
 * \date   December, 2018
 * 
 * \brief Final Exam for the course Financial Computing with C++. 
 *
 */

//! Problems for the course.   
namespace prb
{
  /** 
   * \defgroup prbData Construction of data curves.
   * 
   * This module contains functions that construct input data curves,
   * such as volatility, forward, and discount, for financial models.
   */
  //@{
  /**
   * Computes forward exchange rate curve in the form:
   * \code
   *   F(t) = dSpotFX *  exp(A*(t-dInitialTime)), t>=dInitialTime,
   * \endcode
   * where the cost-of-carry rate \a A is obtained by a least square fit to
   * the market cost-of-carry rates.
   * 
   * \param dSpotFX The spot exchange rate. 
   * \param rMaturities The vector of maturities for discount factors. 
   * \param rDomesticDiscountFactors The vector of domestic discount factors. 
   * \param rForeignDiscountFactors The vector of foreign discount factors. 
   * \param dInitialTime The initial time. 
   * 
   * \return The forward exchange rate curve obtained by least-square fit of 
   * cost-of-carry rates.
   */
  cfl::Function forwardFXFit(double dSpotFX, 
			     const std::vector<double> & rMaturities, 
			     const std::vector<double> & rDomesticDiscountFactors,
			     const std::vector<double> & rForeignDiscountFactors, 
			     double dInitialTime);
  //@}

  /**
   * \defgroup prbAssetStd Standard and barrier options on a single stock. 
   *
   * This module contains functions that compute prices of standard
   * and barrier options in cfl::AssetModel.
   */
  //@{

  /** 
   * Computes the value of the <strong>down-and-rebate or up-and-out barrier
   * option</strong>. There are two types of barrier events:
   * <ol>
   * <li> <em>The lower barrier event</em> is taking place at the first time
   * when the price of the stock is below the lower barrier 
   * \a dLowerBarrier. In this case, the option pays the notional amount 
   * \a dNotional.
   * <li> <em>The upper barrier event</em> is taking place at the
   * first time when the price of the stock is above the upper barrier
   * \a dUpperBarrier. In this case, the option expires worthless.
   * </ol>
   * After any of the barrier events the option is terminated. If neither of
   * the barriers has been crossed, then the option expires worthless.  
   * 
   * \param dNotional The notional amount.  
   * \param dLowerBarrier The lower barrier.  
   * \param dUpperBarrier The upper barrier.  
   * \param rLowerBarrierTimes The lower barrier times. 
   * \param rUpperBarrierTimes The upper barrier times. 
   * \param rModel Reference to implementation of cfl::AssetModel. 
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.  
   */
  cfl::MultiFunction 
  downAndRebateOrUpAndOut(double dNotional, double dLowerBarrier, 
			  double dUpperBarrier, 
			  const std::vector<double> & rLowerBarrierTimes,
			  const std::vector<double> & rUpperBarrierTimes, 
			  cfl::AssetModel & rModel);
  //@}

  /**
   * \defgroup prbInterestRateStd Standard and barrier options on interest rates.
   *
   * This module contains functions that compute prices of standard
   * and barrier options in cfl::InterestRateModel.
   */
  //@{
  /** 
   * Computes the price of <strong>cap on swap rate set in arrears</strong>. 
   * In this contract, the payments occur at the times
   * \code
   * t(i) = rModel.initialTime() + i*rCap.period,  
   * i=1,...,rCap.numberOfPayments,
   * \endcode
   * and given by 
   * \code
   * rCap.notional*rCap.period*max(R(t(i)) - rCap.rate,0.)
   * \endcode
   * where <code>R(t(i))</code> is the swap rate computed at
   * <code>t(i)</code>
   * in the swap contract with number of payments \a iSwapPayments
   * and the interval between the payments \a dSwapPeriod. 
   * 
   * \param rCap The parameters of interest rate cap. 
   * \param dSwapPeriod The period in the interest rate swap. 
   * \param iSwapPayments The number of payments in interest rate swap. 
   * \param rModel Reference to implementation of cfl::InterestRateModel.
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.
   */
  cfl::MultiFunction
  capOnSwapRateArrears(const cfl::Data::CashFlow & rCap, 
		       double dSwapPeriod, unsigned iSwapPayments, 
		       cfl::InterestRateModel & rModel);
  //@}

  /** 
   * \defgroup prbPath Path dependent options.  
   *
   * This module contains functions that compute prices of path
   * dependent options in cfl::AssetModel and cfl::InterestRateModel.
   */
  //@{
  /** 
   * Computes the value of the <strong>deferred start average rate
   * swap</strong>.  The swap starts at \a dStartTime with the fixed
   * rate 
   * \f[
   * \frac1{1+M}(R_0 + \sum_{i=1}^M R_i),
   * \f]
   * where \f$R_0=\f$ \a rSwap.rate and \f$R_i\f$ is the market 
   * swap rate at reset time \f$s_i\f$ for the contract with 
   * same parameters as the underlying swap, \f$i=1,\dots,M\f$. 
   * 
   * \param rResetTimes The reset times for the computation of fixed
   * rate (\f$(s_i)_{i=1,\dots,M}\f$); 
   * \a rResetTimes.front() > \a rModel.initialTime(). 
   * \param dStartTime The start time for the swap; 
   * \a rResetTimes.back() < \a dStartTime.
   * \param rSwap The parameters of the underlying swap. 
   * \param rModel Reference to implementation of cfl::InterestRateModel.
   * 
   * \return The price of the option as the function of the initial
   * values of the state processes in the model.
   */

  cfl::MultiFunction 
  deferredStartAverageRateSwap(const std::vector<double> & rResetTimes,
			       double dStartTime, 
			       const cfl::Data::Swap & rSwap,
			       cfl::InterestRateModel & rModel);
  //@}
}
#endif // of __Exam_hpp__
